Package: toolero
Title: A Toolkit for Research Workflows
Version: 0.3.0
Authors@R: 
    person("Erwin", "Lares", , "erwin.lares@wisc.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3284-828X"))
Description: Provides utility functions to help researchers implement best
    practices for their coding projects. Includes tools for reading and
    cleaning data files, initializing R projects with a standard folder
    structure, creating Quarto documents from a reproducible template,
    detecting the execution context across interactive, Quarto, and
    script-based workflows, and splitting data frames into group-level
    output files.
License: MIT + file LICENSE
Depends: R (>= 4.2.0)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0),
Config/testthat/edition: 3
Imports: cli, fs, glue, janitor, purrr, readr, renv, tibble, usethis,
        yaml, rlang, rvest, xml2, quarto, withr
URL: https://github.com/erwinlares/toolero,
        https://erwinlares.github.io/toolero/
BugReports: https://github.com/erwinlares/toolero/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-27 15:23:42 UTC; lares
Author: Erwin Lares [aut, cre] (ORCID: <https://orcid.org/0000-0002-3284-828X>)
Maintainer: Erwin Lares <erwin.lares@wisc.edu>
Repository: CRAN
Date/Publication: 2026-04-27 18:40:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:09:40 UTC; unix

Package: toolero
Title: A Toolkit for Research Workflows
Version: 0.3.0
Authors@R: 
    person("Erwin", "Lares", , "erwin.lares@wisc.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3284-828X"))
Description: Provides utility functions to help researchers implement best
    practices for their coding projects. Includes tools for reading and
    cleaning data files, initializing R projects with a standard folder
    structure, creating Quarto documents from a reproducible template,
    detecting the execution context across interactive, Quarto, and
    script-based workflows, and splitting data frames into group-level
    output files.
License: MIT + file LICENSE
Depends: R (>= 4.2.0)
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0),
Config/testthat/edition: 3
Imports: cli, fs, glue, janitor, purrr, readr, renv, tibble, usethis,
        yaml, rlang, rvest, xml2, quarto, withr
URL: https://github.com/erwinlares/toolero,
        https://erwinlares.github.io/toolero/
BugReports: https://github.com/erwinlares/toolero/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-27 15:23:42 UTC; lares
Author: Erwin Lares [aut, cre] (ORCID: <https://orcid.org/0000-0002-3284-828X>)
Maintainer: Erwin Lares <erwin.lares@wisc.edu>
Repository: CRAN
Date/Publication: 2026-04-27 18:40:02 UTC
Built: R 4.6.0; ; 2026-04-27 21:40:06 UTC; unix

# Generated by roxygen2: do not edit by hand

export(create_qmd)
export(detect_execution_context)
export(generate_kb_xml)
export(init_project)
export(read_clean_csv)
export(write_by_group)
importFrom(rlang,"%||%")

# toolero
<img src="man/figures/logo.png" align="right" height="139" alt="toolero package logo"/>

<!-- badges: start -->
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.19457647.svg)](https://doi.org/10.5281/zenodo.19457647)
[![R-CMD-check](https://github.com/erwinlares/toolero/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/erwinlares/toolero/actions/workflows/R-CMD-check.yaml)
[![CRAN status](https://www.r-pkg.org/badges/version/toolero)](https://CRAN.R-project.org/package=toolero)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/toolero)](https://cran.r-project.org/package=toolero)
<!-- badges: end -->

`toolero` is an R package designed to help researchers implement best practices
for their coding projects. It provides a small set of opinionated, practical
functions that reduce friction at the start of a project and during day-to-day
data work.

## Installation

You can install `toolero` from CRAN:

``` r
install.packages("toolero")
```

Or install the development version from GitHub:

``` r
# install.packages("pak")
pak::pak("erwinlares/toolero")
```

## Functions

### `init_project()`

Creates a new R project with a standard folder structure suited for research
workflows. Optionally initializes `renv` for package management and `git` for
version control.

``` r
library(toolero)

# Create a project with the standard folder structure
init_project(path = "~/Documents/my-project")

# Add extra folders
init_project(path = "~/Documents/my-project",
             extra_folders = c("notebooks", "presentations"))

# Skip renv and git
init_project(path = "~/Documents/my-project", use_renv = FALSE, use_git = FALSE)
```

The default folder structure includes: `data/`, `data-raw/`, `R/`, `scripts/`,
`plots/`, `images/`, `results/`, and `docs/`.

### `create_qmd()`

Scaffolds a new Quarto document from a reproducible template, including a
sample dataset, UW-Madison branded assets, and an optional post-render purl
hook that extracts R code from the rendered document into a companion `.R`
file. Optionally pre-populates the YAML header from a user-supplied YAML
config file.

``` r
library(toolero)

# Create a document with placeholder YAML
create_qmd(path = "~/Documents/my-project", filename = "analysis.qmd")

# Create without the purl hook
create_qmd(path = "~/Documents/my-project", filename = "report.qmd",
           use_purl = FALSE)

# Pre-populate YAML from a personal config file
create_qmd(path = "~/Documents/my-project", filename = "analysis.qmd",
           yaml_data = "~/my_config.yml")
```

### `read_clean_csv()`

Reads a CSV file and cleans the column names in one step, producing a
tidyverse-friendly tibble.

``` r
library(toolero)

data <- read_clean_csv("path/to/file.csv")

# Show column type messages
data <- read_clean_csv("path/to/file.csv", verbose = TRUE)
```

### `detect_execution_context()`

Identifies which of three execution environments the code is currently running
in: an interactive R session, a `quarto render` call, or a plain `Rscript`
invocation. Returns one of `"interactive"`, `"quarto"`, or `"rscript"`.

``` r
library(toolero)

context <- detect_execution_context()

input_file <- switch(context,
  interactive = "data/sample.csv",
  quarto      = params$input_file,
  rscript     = commandArgs(trailingOnly = TRUE)[1]
)
```

### `write_by_group()`

Splits a data frame by a single grouping column and writes each group to a
separate CSV file. Filenames are derived from sanitized group values —
converted to lowercase with spaces and special characters replaced by dashes.
Optionally writes a `manifest.csv` listing output files, group values, and
row counts.

``` r
library(toolero)

# Load the bundled sample dataset
sample_path <- system.file("templates", "sample.csv", package = "toolero")
penguins    <- read_clean_csv(sample_path)

# Split by species
write_by_group(penguins, group_col = "species", output_dir = tempdir())

# Also write a manifest
write_by_group(penguins, group_col = "species",
               output_dir = tempdir(), manifest = TRUE)
```

### `generate_kb_xml()`

Produces a UW-Madison Knowledge Base importable XML file from a rendered
Quarto document. Re-renders the source `.qmd` with all assets embedded,
extracts the HTML body, and wraps it in the KB XML structure along with
metadata drawn from the document's YAML header — `title` → `kb_title`,
`description` → `kb_summary`, `categories` → `kb_keywords`.

``` r
library(toolero)

generate_kb_xml(
  html_path  = "docs/analysis.html",
  output_dir = "exports"
)
```

When importing the resulting XML into the KB, check the
*Decode HTML entity in body content* option.

## Citation

To cite `toolero` in publications:

``` r
citation("toolero")
```

## License

MIT © Erwin Lares

qmd_files <- fs::dir_ls(getwd(), glob = "*.qmd")

for (input in qmd_files) {
    output <- fs::path("R", fs::path_ext_set(fs::path_file(input), "R"))
    knitr::purl(input, output = output, documentation = 1)
}

#' Create a new Quarto document from a template
#'
#' Creates a new Quarto document in the specified directory, along with a
#' sample dataset and UW-Madison branded assets. Optionally pre-populates
#' the YAML header with user-supplied metadata.
#'
#' @param path A string or `NULL`. Path to the directory where the document
#'   will be created. If `NULL`, the user must supply a path explicitly.
#' @param filename A string. Name of the generated `.qmd` file. Defaults to
#'   `"analysis.qmd"`.
#' @param yaml_data A string or `NULL`. Path to a YAML file containing
#'   metadata to pre-populate the document header. If `NULL` (the default),
#'   the template is copied as-is with placeholder prompts intact.
#' @param overwrite A logical. Whether to overwrite existing files. Defaults
#'   to `FALSE`.
#'
#' @return Invisibly returns `path`.
#'
#' @details
#' `create_qmd()` performs the following steps:
#'
#' 1. Validates that `path` exists.
#' 2. Creates a `data/` folder under `path` and copies `sample.csv` there.
#' 3. Checks for `assets/styles.css` and `assets/header.html` - creates the
#'    `assets/` folder if needed and copies both from the package.
#' 4. Copies the template `.qmd` to `path/filename`.
#' 5. If `yaml_data` is provided, reads the YAML file and substitutes values
#'    into the document header.
#'
#' Note: `path` has no default value. Always supply an explicit path to avoid
#' writing files to unexpected locations. Use `tempdir()` for temporary output
#' during testing or exploration.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Create with placeholder YAML in a temp directory
#' create_qmd(path = tempdir())
#'
#' # Create with a custom filename
#' create_qmd(path = tempdir(), filename = "report.qmd", overwrite = TRUE)
#'
#' # Create with pre-populated YAML
#' yaml_file <- tempfile(fileext = ".yml")
#' writeLines("author:\n  - name: 'Your Name'", yaml_file)
#' create_qmd(path = tempdir(), yaml_data = yaml_file, overwrite = TRUE)
#' }
create_qmd <- function(
        path = NULL,
        filename = "analysis.qmd",
        yaml_data = NULL,
        overwrite = FALSE) {

    # -- 0. Validate path -------------------------------------------------------
    if (is.null(path)) {
        cli::cli_abort(
            "{.arg path} must be supplied. Use {.code tempdir()} for temporary
       output or provide an explicit path."
        )
    }

    # -- 1. Validate path -------------------------------------------------------
    if (!fs::dir_exists(path)) {
        cli::cli_abort(
            "Directory {.path {path}} does not exist.
       Create it first or choose an existing path."
        )
    }

    # -- 2. Create data/ and copy sample.csv ------------------------------------
    data_dir <- fs::path(path, "data")
    fs::dir_create(data_dir)

    sample_src <- system.file(
        "templates", "sample.csv",
        package = "toolero",
        mustWork = TRUE
    )
    sample_dst <- fs::path(data_dir, "sample.csv")

    if (!fs::file_exists(sample_dst) || overwrite) {
        fs::file_copy(sample_src, sample_dst, overwrite = overwrite)
        cli::cli_alert_success("Created {.path {sample_dst}}")
    } else {
        cli::cli_alert_info("Skipping {.path {sample_dst}} - already exists.")
    }

    # -- 3. Create assets/ and copy styles.css and header.html -----------------
    assets_dir <- fs::path(path, "assets")
    fs::dir_create(assets_dir)

    assets <- c("styles.css", "header.html")

    for (asset in assets) {
        asset_src <- system.file(
            "assets", asset,
            package = "toolero",
            mustWork = TRUE
        )
        asset_dst <- fs::path(assets_dir, asset)

        if (!fs::file_exists(asset_dst) || overwrite) {
            fs::file_copy(asset_src, asset_dst, overwrite = overwrite)
            cli::cli_alert_success("Created {.path {asset_dst}}")
        } else {
            cli::cli_alert_info("Skipping {.path {asset_dst}} - already exists.")
        }
    }

    # -- 4. Copy template .qmd --------------------------------------------------
    qmd_src <- system.file(
        "templates", "example.qmd",
        package = "toolero",
        mustWork = TRUE
    )
    qmd_dst <- fs::path(path, filename)

    if (fs::file_exists(qmd_dst) && !overwrite) {
        cli::cli_abort(
            "{.path {qmd_dst}} already exists.
       Use {.code overwrite = TRUE} to replace it."
        )
    }

    qmd_content <- readr::read_file(qmd_src)

    # -- 5. Substitute YAML if yaml_data is provided ----------------------------
    if (!is.null(yaml_data)) {
        if (!fs::file_exists(yaml_data)) {
            cli::cli_abort(
                "yaml_data file {.path {yaml_data}} does not exist."
            )
        }

        user_yaml <- yaml::read_yaml(yaml_data)
        qmd_content <- substitute_yaml(qmd_content, user_yaml)
    }

    readr::write_file(qmd_content, qmd_dst)
    cli::cli_alert_success("Created {.path {qmd_dst}}")

    invisible(path)
}


# -- Helper: substitute YAML values into template ----------------------------

substitute_yaml <- function(qmd_content, user_yaml) {

    # Normalize line endings to \n regardless of platform
    qmd_content <- gsub("\r\n", "\n", qmd_content, fixed = TRUE)

    yaml_pattern <- "(?s)^---\\n(.+?)\\n---"
    yaml_match <- regmatches(
        qmd_content,
        regexpr(yaml_pattern, qmd_content, perl = TRUE)
    )

    if (length(yaml_match) == 0) {
        cli::cli_warn("No YAML header found in template. Skipping substitution.")
        return(qmd_content)
    }

    # Parse template YAML
    template_yaml <- yaml::yaml.load(yaml_match)

    # Directly overwrite keys present in user_yaml
    for (key in names(user_yaml)) {
        template_yaml[[key]] <- user_yaml[[key]]
    }

    # Serialize and reconstruct
    merged_yaml_str <- yaml::as.yaml(template_yaml)
    new_header <- paste0("---\n", merged_yaml_str, "---")

    sub(yaml_pattern, new_header, qmd_content, perl = TRUE)
}

# tests/testthat/test-create_qmd.R

# Helper: write a minimal yaml_data file to a temp location
make_yaml_config <- function(path) {
  yaml_content <- "
author:
  - name: 'Erwin Lares'
    affiliation: 'RCI, UW-Madison'
    orcid: '0000-0002-3284-828X'
    email: 'erwin.lares@wisc.edu'
"
  readr::write_file(yaml_content, path)
}

# -- Happy path ---------------------------------------------------------------

test_that("creates the named .qmd in the specified path", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_true(fs::file_exists(fs::path(tmp, "analysis.qmd")))
})

test_that("creates data/ folder and copies sample.csv", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_true(fs::dir_exists(fs::path(tmp, "data")))
  expect_true(fs::file_exists(fs::path(tmp, "data", "sample.csv")))
})

test_that("creates assets/ folder and copies styles.css and header.html", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_true(fs::dir_exists(fs::path(tmp, "assets")))
  expect_true(fs::file_exists(fs::path(tmp, "assets", "styles.css")))
  expect_true(fs::file_exists(fs::path(tmp, "assets", "header.html")))
})

test_that("returns path invisibly", {
  tmp <- withr::local_tempdir()
  result <- create_qmd(path = tmp, filename = "analysis.qmd")
  expect_equal(result, tmp)
})

test_that("respects custom filename argument", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "report.qmd")
  expect_true(fs::file_exists(fs::path(tmp, "report.qmd")))
  expect_false(fs::file_exists(fs::path(tmp, "analysis.qmd")))
})

test_that("pre-populates YAML when yaml_data is provided", {
  tmp <- withr::local_tempdir()
  yaml_file <- withr::local_tempfile(fileext = ".yml")
  make_yaml_config(yaml_file)

  create_qmd(path = tmp, filename = "analysis.qmd", yaml_data = yaml_file)

  qmd_content <- readr::read_file(fs::path(tmp, "analysis.qmd"))
  expect_true(grepl("Erwin Lares", qmd_content, fixed = TRUE))
  expect_true(grepl("RCI, UW-Madison", qmd_content, fixed = TRUE))
  expect_true(grepl("0000-0002-3284-828X", qmd_content, fixed = TRUE))
})

test_that("copies template as-is with placeholders when yaml_data is NULL", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")

  qmd_content <- readr::read_file(fs::path(tmp, "analysis.qmd"))
  expect_true(grepl("Your Name", qmd_content, fixed = TRUE))
  expect_true(grepl("Your Document Title", qmd_content, fixed = TRUE))
})

# -- use_purl -----------------------------------------------------------------

test_that("creates _quarto.yml when use_purl = TRUE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = TRUE)
  expect_true(fs::file_exists(fs::path(tmp, "_quarto.yml")))
})

test_that("_quarto.yml contains post-render hook", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = TRUE)
  content <- readr::read_file(fs::path(tmp, "_quarto.yml"))
  expect_true(grepl("post-render", content, fixed = TRUE))
  expect_true(grepl("purl.R", content, fixed = TRUE))
})

test_that("creates purl.R when use_purl = TRUE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = TRUE)
  expect_true(fs::file_exists(fs::path(tmp, "R", "purl.R")))
})

test_that("purl.R uses glob-based qmd scanning", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = TRUE)
  content <- readr::read_file(fs::path(tmp, "R", "purl.R"))
  expect_true(grepl("dir_ls", content, fixed = TRUE))
})

test_that("does not create _quarto.yml when use_purl = FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = FALSE)
  expect_false(fs::file_exists(fs::path(tmp, "_quarto.yml")))
})

test_that("does not create purl.R when use_purl = FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = FALSE)
  expect_false(fs::file_exists(fs::path(tmp, "R", "purl.R")))
})

test_that("skips existing _quarto.yml without erroring when overwrite is FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd", use_purl = TRUE)
  expect_no_error(
    create_qmd(path = tmp, filename = "report.qmd", use_purl = TRUE)
  )
})

# -- Edge cases ---------------------------------------------------------------

test_that("errors informatively when filename is NULL", {
  tmp <- withr::local_tempdir()
  expect_error(
    create_qmd(path = tmp),
    "filename"
  )
})

test_that("errors informatively when path does not exist", {
  expect_error(
    create_qmd(path = "/this/does/not/exist", filename = "analysis.qmd"),
    "does not exist"
  )
})

test_that("errors informatively when yaml_data path does not exist", {
  tmp <- withr::local_tempdir()
  expect_error(
    create_qmd(path = tmp, filename = "analysis.qmd",
               yaml_data = "/no/such/file.yml"),
    "does not exist"
  )
})

test_that("errors when .qmd already exists and overwrite is FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_error(
    create_qmd(path = tmp, filename = "analysis.qmd"),
    "already exists"
  )
})

test_that("overwrites .qmd when overwrite is TRUE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_no_error(
    create_qmd(path = tmp, filename = "analysis.qmd", overwrite = TRUE)
  )
})

test_that("skips existing assets without erroring when overwrite is FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_no_error(
    create_qmd(path = tmp, filename = "report.qmd")
  )
})

test_that("skips existing sample.csv without erroring when overwrite is FALSE", {
  tmp <- withr::local_tempdir()
  create_qmd(path = tmp, filename = "analysis.qmd")
  expect_no_error(
    create_qmd(path = tmp, filename = "report.qmd")
  )
})

# -- substitute_yaml() helper -------------------------------------------------

test_that("substitute_yaml() merges user values into template YAML", {
  template <- "---\ntitle: 'Your Document Title'\nauthor:\n  - name: 'Your Name'\n---\n\nBody text."
  user_yaml <- list(title = "My Real Title")

  result <- substitute_yaml(template, user_yaml)
  expect_true(grepl("My Real Title", result, fixed = TRUE))
})

test_that("substitute_yaml() warns and returns content unchanged when no YAML header found", {
  content <- "No YAML here, just body text."
  user_yaml <- list(title = "My Title")

  expect_warning(
    result <- substitute_yaml(content, user_yaml),
    "No YAML header found"
  )
  expect_equal(result, content)
})

test_that("substitute_yaml() serializes logicals as true/false not yes/no", {
  template <- "---\ntoc: true\nnumber-sections: true\nembed-resources: false\n---\n\nBody."
  user_yaml <- list(title = "My Title")

  result <- substitute_yaml(template, user_yaml)
  expect_true(grepl("true", result, fixed = TRUE))
  expect_false(grepl("yes", result, fixed = TRUE))
  expect_false(grepl("no", result, fixed = TRUE))
})

#' Read and clean a CSV file
#'
#' `read_clean_csv()` reads a CSV file and cleans the column names in one step.
#' It leverages `readr::read_csv()` for reading and `janitor::clean_names()` for
#' making column names tidyverse-friendly (lowercase, no spaces, no special
#' characters). By default, column type messages are suppressed. Set
#' `verbose = TRUE` to display them.
#'
#' @param file_path A character string with the path to the CSV file.
#' @param verbose Logical. If `TRUE`, displays column type messages from
#'   `readr::read_csv()`. Defaults to `FALSE`.
#'
#' @return A tibble with clean column names.
#' @export
#'
#' @examples
#' \donttest{
#' # Read and clean a CSV file silently
#' sample_path <- system.file("templates", "sample.csv", package = "toolero")
#' data <- read_clean_csv(sample_path)
#'
#' # Show column type messages
#' data <- read_clean_csv(sample_path, verbose = TRUE)
#' }
read_clean_csv <- function(file_path, verbose = FALSE) {
    readr::read_csv(file_path, show_col_types = verbose) |>
        janitor::clean_names()
}

test_that("read_clean_csv() returns a tibble", {
    tmp <- tempfile(fileext = ".csv")
    writeLines("First Name,Last Name,Age\nJane,Doe,30", tmp)

    result <- read_clean_csv(tmp)

    expect_s3_class(result, "tbl_df")
})

test_that("read_clean_csv() cleans column names", {
    tmp <- tempfile(fileext = ".csv")
    writeLines("First Name,Last Name,Age\nJane,Doe,30", tmp)

    result <- read_clean_csv(tmp)

    expect_named(result, c("first_name", "last_name", "age"))
})

test_that("read_clean_csv() reads data correctly", {
    tmp <- tempfile(fileext = ".csv")
    writeLines("First Name,Last Name,Age\nJane,Doe,30", tmp)

    result <- read_clean_csv(tmp)

    expect_equal(nrow(result), 1)
    expect_equal(ncol(result), 3)
})

test_that("read_clean_csv() errors on non-existent file", {
    expect_error(read_clean_csv("non_existent_file.csv"))
})

test_that("read_clean_csv() verbose argument works", {
    tmp <- tempfile(fileext = ".csv")
    writeLines("First Name,Last Name,Age\nJane,Doe,30", tmp)

    # should run without error in both modes
    expect_no_error(read_clean_csv(tmp, verbose = FALSE))
    expect_no_error(read_clean_csv(tmp, verbose = TRUE))
})

#' Initialize a new R project with a standard folder structure
#'
#' `init_project()` creates a new R project at the given path with an
#' opinionated folder structure suited for research workflows. It optionally
#' initializes `renv` for package management and git for version control.
#'
#' @param file_path A character string with the path and name of the new
#'   project (e.g., `"~/Documents/my-project"`).
#' @param use_renv Logical. If `TRUE`, initializes `renv` in the new project.
#'   Defaults to `TRUE`.
#' @param use_git Logical. If `TRUE`, initializes a git repository in the new
#'   project. Defaults to `TRUE`.
#' @param extra_folders A character vector of additional folder names to create
#'   inside the project. Defaults to `NULL`.
#' @param open Logical. If `TRUE`, opens the new project in RStudio after
#'   creation. Defaults to `TRUE`.
#' @param uw_branding Logical. If `TRUE`, creates an `assets/` folder and
#'   populates it with UW-Madison RCI branding files (`styles.css`,
#'   `header.html`, `rci-banner.png`). Defaults to `FALSE`.
#'
#' @return Called for its side effects. Does not return a value.
#' @export
#'
#' @examples
#' \donttest{
#' init_project(file_path = file.path(tempdir(), "project1"),
#'              use_renv = FALSE, use_git = FALSE)
#'
#' init_project(file_path = file.path(tempdir(), "project2"),
#'              uw_branding = TRUE, use_renv = FALSE, use_git = FALSE)
#'
#' init_project(file_path = file.path(tempdir(), "project3"),
#'              extra_folders = c("notebooks"),
#'              use_renv = FALSE, use_git = FALSE)
#' }

init_project <- function(file_path,
                         use_renv = TRUE,
                         use_git = TRUE,
                         extra_folders = NULL,
                         open = FALSE,
                         uw_branding = FALSE) {

    withr::with_dir(getwd(), {

        # 1. create the RStudio project
        usethis::create_project(file_path, open = FALSE)

        # 2. create standard folders
        standard_folders <- c("data", "data-raw", "images", "plots",
                              "results", "scripts", "docs", "R")
        purrr::walk(standard_folders,
                    \(folder) fs::dir_create(glue::glue("{file_path}/{folder}")))

        # 3. create any extra folders
        if (!is.null(extra_folders)) {
            purrr::walk(extra_folders,
                        \(folder) fs::dir_create(glue::glue("{file_path}/{folder}")))
        }

        # 4. copy UW-Madison RCI branding files into assets/
        if (uw_branding) {
            assets_dir <- file.path(file_path, "assets")
            fs::dir_create(assets_dir)
            branding_files <- c("styles.css", "header.html", "rci-banner.png")
            purrr::walk(branding_files, \(f) {
                fs::file_copy(
                    system.file("extdata", f, package = "toolero"),
                    file.path(assets_dir, f)
                )
            })
        }

        # 5. initialize renv
        if (use_renv) renv::init(project = file_path, restart = FALSE)

        # 6. initialize git
        if (use_git) usethis::use_git(message = "initial commit")

        # 7. open the project in RStudio
        if (open) usethis::proj_activate(file_path)

    })
}

test_that("init_project() creates the standard folder structure", {
    tmp <- file.path(tempdir(), "test-project")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE)

    standard_folders <- c("data", "data-raw", "images", "plots", "results", "scripts", "docs", "R")
    purrr::walk(standard_folders, \(folder) {
        expect_true(fs::dir_exists(file.path(tmp, folder)))
    })
})

test_that("init_project() creates extra folders when provided", {
    tmp <- file.path(tempdir(), "test-project-extra")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE,
                 extra_folders = c("notebooks", "presentations"))

    expect_true(fs::dir_exists(file.path(tmp, "notebooks")))
    expect_true(fs::dir_exists(file.path(tmp, "presentations")))
})

test_that("init_project() does not create extra folders when extra_folders is NULL", {
    tmp <- file.path(tempdir(), "test-project-null")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE)

    expect_false(fs::dir_exists(file.path(tmp, "notebooks")))
})

test_that("init_project() creates a valid project structure", {
    tmp <- file.path(tempdir(), "test-project-rproj")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE)

    expect_true(fs::dir_exists(tmp))
})

test_that("init_project() creates assets/ folder when uw_branding = TRUE", {
    tmp <- file.path(tempdir(), "test-project-branding")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE,
                 uw_branding = TRUE)

    expect_true(fs::dir_exists(file.path(tmp, "assets")))
})

test_that("init_project() copies branding files when uw_branding = TRUE", {
    tmp <- file.path(tempdir(), "test-project-branding-files")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE,
                 uw_branding = TRUE)

    expect_true(fs::file_exists(file.path(tmp, "assets", "styles.css")))
    expect_true(fs::file_exists(file.path(tmp, "assets", "header.html")))
    expect_true(fs::file_exists(file.path(tmp, "assets", "rci-banner.png")))
})

test_that("init_project() does not create assets/ folder when uw_branding = FALSE", {
    tmp <- file.path(tempdir(), "test-project-no-branding")
    on.exit(fs::dir_delete(tmp), add = TRUE)

    init_project(tmp, use_renv = FALSE, use_git = FALSE, open = FALSE,
                 uw_branding = FALSE)

    expect_false(fs::dir_exists(file.path(tmp, "assets")))
})

#' Split a data frame by a grouping column and write each group to a CSV file
#'
#' Splits a data frame by a single grouping column and writes each group to
#' a separate CSV file. Optionally writes a manifest file listing the output
#' files, their group values, and row counts.
#'
#' @param data A data frame or tibble to split and save.
#' @param group_col A string. The name of the column to group by.
#' @param output_dir A string or `NULL`. Path to the directory where output
#'   files will be written. Created if it does not exist. If `NULL`, the
#'   user must supply a path explicitly.
#' @param manifest A logical. Whether to write a `manifest.csv` file to
#'   `output_dir` listing the output files, group values, and row counts.
#'   Defaults to `FALSE`.
#'
#' @return Invisibly returns `output_dir`.
#'
#' @details
#' Output filenames are derived from the group values of `group_col`.
#' Values are sanitized before use as filenames: converted to lowercase,
#' spaces and special characters replaced with `_`, consecutive underscores
#' collapsed, and leading/trailing underscores stripped.
#'
#' If `manifest = TRUE`, a `manifest.csv` is written to `output_dir`
#' containing three columns: `group_value`, `n_rows`, and `file_path`.
#'
#' Note: `output_dir` has no default value. Always supply an explicit path
#' to avoid writing files to unexpected locations. Use `tempdir()` for
#' temporary output during testing or exploration.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Split a small data frame by group and write to a temp directory
#' data <- data.frame(
#'   species = c("Adelie", "Adelie", "Gentoo"),
#'   mass    = c(3750, 3800, 5000)
#' )
#' write_by_group(data, group_col = "species", output_dir = tempdir())
#'
#' # Same but also write a manifest
#' write_by_group(data, group_col = "species",
#'                output_dir = tempdir(), manifest = TRUE)
#' }
write_by_group <- function(
        data,
        group_col,
        output_dir = NULL,
        manifest = FALSE) {

    # -- 0. Validate output_dir -------------------------------------------------
    if (is.null(output_dir)) {
        cli::cli_abort(
            "{.arg output_dir} must be supplied. Use {.code tempdir()} for temporary
       output or provide an explicit path."
        )
    }
    # -- 1. Validate data -------------------------------------------------------
    if (!is.data.frame(data)) {
        cli::cli_abort(
            "{.arg data} must be a data frame or tibble, not {.cls {class(data)}}."
        )
    }

    # -- 2. Validate group_col --------------------------------------------------
    if (!group_col %in% names(data)) {
        cli::cli_abort(
            "Column {.val {group_col}} not found in {.arg data}.
       Available columns: {.val {names(data)}}."
        )
    }

    # -- 3. Create output_dir if needed -----------------------------------------
    fs::dir_create(output_dir)

    # -- 4. Split data by group_col ---------------------------------------------
    groups <- split(data, data[[group_col]])

    # -- 5 & 6. Sanitize names and write each group to CSV ----------------------
    manifest_rows <- purrr::map(names(groups), function(group_value) {
        sanitized <- sanitize_filename(group_value)
        file_path  <- fs::path(output_dir, paste0(sanitized, ".csv"))

        readr::write_csv(groups[[group_value]], file_path)
        cli::cli_alert_success(
            "Written {.val {group_value}} ({nrow(groups[[group_value]])} rows) to {.path {file_path}}"
        )

        tibble::tibble(
            group_value = group_value,
            n_rows      = nrow(groups[[group_value]]),
            file_path   = as.character(file_path)
        )
    })

    # -- 7. Write manifest if requested -----------------------------------------
    if (manifest) {
        manifest_df   <- purrr::list_rbind(manifest_rows)
        manifest_path <- fs::path(output_dir, "manifest.csv")
        readr::write_csv(manifest_df, manifest_path)
        cli::cli_alert_success("Manifest written to {.path {manifest_path}}")
    }

    invisible(output_dir)
}


# -- Helper: sanitize a string for use as a filename -------------------------

sanitize_filename <- function(x) {
    x |>
        tolower() |>
        gsub(pattern = "[^a-z0-9]+", replacement = "_", x = _) |>
        gsub(pattern = "^_+|_+$",    replacement = "",  x = _)
}

# tests/testthat/test-write_by_group.R

# Helper: minimal test data
make_test_data <- function() {
    tibble::tibble(
        species   = c("Adelie", "Adelie", "Gentoo", "Gentoo", "Chinstrap"),
        body_mass = c(3750, 3800, 5000, 4900, 3500)
    )
}

# -- Happy path ---------------------------------------------------------------

test_that("creates one CSV file per group", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp)

    expect_true(fs::file_exists(fs::path(tmp, "adelie.csv")))
    expect_true(fs::file_exists(fs::path(tmp, "gentoo.csv")))
    expect_true(fs::file_exists(fs::path(tmp, "chinstrap.csv")))
})

test_that("CSV filenames match sanitized group values", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp)

    written_files <- fs::path_file(fs::dir_ls(tmp, glob = "*.csv"))
    expect_setequal(written_files, c("adelie.csv", "gentoo.csv", "chinstrap.csv"))
})

test_that("each CSV contains only rows for that group", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp)

    adelie <- readr::read_csv(fs::path(tmp, "adelie.csv"), show_col_types = FALSE)
    expect_equal(nrow(adelie), 2)
    expect_true(all(adelie$species == "Adelie"))

    gentoo <- readr::read_csv(fs::path(tmp, "gentoo.csv"), show_col_types = FALSE)
    expect_equal(nrow(gentoo), 2)
    expect_true(all(gentoo$species == "Gentoo"))

    chinstrap <- readr::read_csv(fs::path(tmp, "chinstrap.csv"), show_col_types = FALSE)
    expect_equal(nrow(chinstrap), 1)
    expect_true(all(chinstrap$species == "Chinstrap"))
})

test_that("returns output_dir invisibly", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    result <- write_by_group(data, group_col = "species", output_dir = tmp)
    expect_equal(result, tmp)
})

test_that("creates output_dir if it does not exist", {
    tmp     <- withr::local_tempdir()
    new_dir <- fs::path(tmp, "new_output")
    data    <- make_test_data()

    expect_false(fs::dir_exists(new_dir))
    write_by_group(data, group_col = "species", output_dir = new_dir)
    expect_true(fs::dir_exists(new_dir))
})

test_that("writes manifest.csv when manifest = TRUE", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp, manifest = TRUE)
    expect_true(fs::file_exists(fs::path(tmp, "manifest.csv")))
})

test_that("manifest contains correct columns", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp, manifest = TRUE)
    manifest <- readr::read_csv(fs::path(tmp, "manifest.csv"), show_col_types = FALSE)

    expect_named(manifest, c("group_value", "n_rows", "file_path"))
})

test_that("manifest row counts match actual group sizes", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp, manifest = TRUE)
    manifest <- readr::read_csv(fs::path(tmp, "manifest.csv"), show_col_types = FALSE)

    adelie_row <- manifest[manifest$group_value == "Adelie", ]
    expect_equal(adelie_row$n_rows, 2)

    chinstrap_row <- manifest[manifest$group_value == "Chinstrap", ]
    expect_equal(chinstrap_row$n_rows, 1)
})

test_that("manifest is not written when manifest = FALSE", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    write_by_group(data, group_col = "species", output_dir = tmp)
    expect_false(fs::file_exists(fs::path(tmp, "manifest.csv")))
})

# -- Edge cases / defensive ---------------------------------------------------

test_that("errors informatively when data is not a data frame", {
    tmp <- withr::local_tempdir()
    expect_error(
        write_by_group(list(a = 1, b = 2), group_col = "a", output_dir = tmp),
        "data frame or tibble"
    )
})

test_that("errors informatively when group_col is not found in data", {
    tmp  <- withr::local_tempdir()
    data <- make_test_data()

    expect_error(
        write_by_group(data, group_col = "island", output_dir = tmp),
        "not found"
    )
})

test_that("sanitizes group values with spaces using dashes in filenames", {
    tmp  <- withr::local_tempdir()
    data <- tibble::tibble(
        group = c("group one", "group one", "group two"),
        value = c(1, 2, 3)
    )

    write_by_group(data, group_col = "group", output_dir = tmp)
    expect_true(fs::file_exists(fs::path(tmp, "group-one.csv")))
    expect_true(fs::file_exists(fs::path(tmp, "group-two.csv")))
})

test_that("sanitizes group values with special characters using dashes in filenames", {
    tmp  <- withr::local_tempdir()
    data <- tibble::tibble(
        group = c("group@one", "group@one", "group#two"),
        value = c(1, 2, 3)
    )

    write_by_group(data, group_col = "group", output_dir = tmp)
    expect_true(fs::file_exists(fs::path(tmp, "group-one.csv")))
    expect_true(fs::file_exists(fs::path(tmp, "group-two.csv")))
})

# -- sanitize_filename() helper -----------------------------------------------

test_that("sanitize_filename() lowercases input", {
    expect_equal(sanitize_filename("Adelie"), "adelie")
})

test_that("sanitize_filename() replaces spaces with dashes", {
    expect_equal(sanitize_filename("group one"), "group-one")
})

test_that("sanitize_filename() replaces special characters with dashes", {
    expect_equal(sanitize_filename("group@one!"), "group-one")
})

test_that("sanitize_filename() collapses consecutive dashes", {
    expect_equal(sanitize_filename("group  one"), "group-one")
})

test_that("sanitize_filename() strips leading and trailing dashes", {
    expect_equal(sanitize_filename("@group@"), "group")
})

#' Generate a KB-importable XML file from a Quarto document
#'
#' Takes a Quarto document and produces an XML file that is directly
#' importable into a UW-Madison Knowledge Base (KB) article. The function
#' re-renders the `.qmd` with `embed-resources: true` so all visual assets
#' are self-contained, extracts the HTML body, and wraps it in the KB XML
#' structure along with metadata drawn from the document's YAML header.
#'
#' @param html_path A string. Path to the rendered HTML file. Used to infer
#'   the output filename and, if `qmd_path` is `NULL`, the location of the
#'   source `.qmd`.
#' @param qmd_path A string or `NULL`. Path to the source `.qmd` file. If
#'   `NULL` (the default), inferred by replacing the `.html` extension of
#'   `html_path` with `.qmd`.
#' @param output_dir A string or `NULL`. Directory where the `.xml` file
#'   will be written. If `NULL` (the default), written to the same directory
#'   as `html_path`.
#'
#' @return Invisibly returns the path to the written `.xml` file.
#'
#' @details
#' `generate_kb_xml()` performs the following steps:
#'
#' 1. Validates that `html_path` exists.
#' 2. Infers `qmd_path` from `html_path` if not supplied, then validates it.
#' 3. Extracts `title`, `description`, and `categories` from the `.qmd` YAML
#'    header and maps them to `kb_title`, `kb_summary`, and `kb_keywords`.
#' 4. Re-renders the `.qmd` in an isolated temporary directory with
#'    `embed-resources: true` so all CSS, images, and JS are self-contained.
#'    The `data/` and `assets/` folders are copied alongside the `.qmd` to
#'    ensure the render succeeds.
#' 5. Extracts the `<body>` from the embedded HTML.
#' 6. Escapes HTML entities in the body for XML compatibility, as required
#'    by the UW-Madison KB import format.
#' 7. Builds the XML structure with `kb_title`, `kb_keywords`, `kb_summary`,
#'    and `kb_body` nodes.
#' 8. Writes the `.xml` file to `output_dir`.
#'
#' Temporary files are managed via `withr::local_tempdir()` and are
#' automatically cleaned up when the function exits, even on error.
#'
#' When importing the resulting XML into the KB, check the
#' \emph{Decode HTML entity in body content} option.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Infer qmd_path automatically, write XML alongside the HTML
#' # generate_kb_xml(html_path = "docs/analysis.html")
#'
#' # Supply qmd_path explicitly and write to a specific output directory
#' # generate_kb_xml(
#' #   html_path  = "docs/analysis.html",
#' #   qmd_path   = "analysis.qmd",
#' #   output_dir = "exports"
#' # )
#' }
generate_kb_xml <- function(
        html_path,
        qmd_path = NULL,
        output_dir = NULL) {

    # -- 1. Validate html_path --------------------------------------------------
    if (!fs::file_exists(html_path)) {
        cli::cli_abort("HTML file {.path {html_path}} does not exist.")
    }

    # -- 2. Infer qmd_path if not provided --------------------------------------
    if (is.null(qmd_path)) {
        qmd_path <- fs::path_ext_set(html_path, "qmd")
    }

    if (!fs::file_exists(qmd_path)) {
        cli::cli_abort(
            "Could not find {.path {qmd_path}}.
       Supply the correct path via {.arg qmd_path}."
        )
    }

    # -- 3. Extract metadata from .qmd YAML -------------------------------------
    meta        <- .extract_qmd_metadata(qmd_path)
    kb_title    <- meta$kb_title
    kb_summary  <- meta$kb_summary
    kb_keywords <- meta$kb_keywords

    # -- 4. Re-render .qmd in isolated temp dir with embed-resources: true ------
    cli::cli_alert_info("Re-rendering {.path {qmd_path}} with embedded resources...")

    # withr::local_tempdir() creates an isolated temp directory that is
    # automatically deleted when the function exits, even on error
    temp_dir  <- withr::local_tempdir()
    temp_qmd  <- fs::path(temp_dir, fs::path_file(qmd_path))
    temp_html <- fs::path_ext_set(temp_qmd, "html")

    fs::file_copy(qmd_path, temp_qmd, overwrite = TRUE)

    data_src <- fs::path(fs::path_dir(qmd_path), "data")
    data_dst <- fs::path(temp_dir, "data")
    if (fs::dir_exists(data_src)) {
        fs::dir_copy(data_src, data_dst, overwrite = TRUE)
    }

    assets_src <- fs::path(fs::path_dir(qmd_path), "assets")
    assets_dst <- fs::path(temp_dir, "assets")
    if (fs::dir_exists(assets_src)) {
        fs::dir_copy(assets_src, assets_dst, overwrite = TRUE)
    }

    quarto::quarto_render(
        input       = temp_qmd,
        quarto_args = c("--embed-resources", "--standalone")
    )

    # -- 5. Extract body from embedded HTML -------------------------------------
    html_doc <- rvest::read_html(temp_html)
    kb_body  <- html_doc |>
        rvest::html_element("body") |>
        as.character()

    # -- 6. Escape HTML entities for XML compatibility --------------------------
    kb_body_escaped <- gsub("<", "&lt;", kb_body, fixed = TRUE)
    kb_body_escaped <- gsub(">", "&gt;", kb_body_escaped, fixed = TRUE)

    # -- 7. Build XML structure -------------------------------------------------
    root   <- xml2::xml_new_root("kb_documents")
    kb_doc <- xml2::xml_add_child(root, "kb_document")

    xml2::xml_add_child(kb_doc, "kb_title",    kb_title)
    xml2::xml_add_child(kb_doc, "kb_keywords", kb_keywords)
    xml2::xml_add_child(kb_doc, "kb_summary",  kb_summary)
    xml2::xml_add_child(kb_doc, "kb_body",     kb_body_escaped)

    # -- 8. Determine output path -----------------------------------------------
    if (is.null(output_dir)) {
        output_dir <- fs::path_dir(html_path)
    }

    fs::dir_create(output_dir)

    output_path <- fs::path(
        output_dir,
        paste0(fs::path_ext_remove(fs::path_file(html_path)), ".xml")
    )

    # -- 9. Write XML -----------------------------------------------------------
    xml2::write_xml(root, output_path)

    cli::cli_alert_success("KB XML written to {.path {output_path}}")
    cli::cli_alert_info(
        "When importing into the KB, check {.emph 'Decode HTML entity in body content'}."
    )

    invisible(output_path)
}


# -- Helper: extract metadata from .qmd YAML header --------------------------

.extract_qmd_metadata <- function(qmd_path) {
    raw_lines  <- readLines(qmd_path, warn = FALSE)
    yaml_end   <- which(raw_lines == "---")[2]
    yaml_block <- paste(raw_lines[2:(yaml_end - 1)], collapse = "\n")
    qmd_yaml   <- yaml::yaml.load(yaml_block)

    list(
        kb_title    = qmd_yaml$title       %||% "",
        kb_summary  = qmd_yaml$description %||% "",
        kb_keywords = if (!is.null(qmd_yaml$categories)) {
            paste(qmd_yaml$categories, collapse = ", ")
        } else {
            ""
        }
    )
}

# tests/testthat/test-generate-kb-xml.R

# Helper: write a minimal .qmd file with known YAML to a temp location
make_test_qmd <- function(path,
                          title       = "Test Title",
                          description = "Test summary",
                          categories  = c("tag1", "tag2")) {
    categories_block <- paste0(
        "categories:\n",
        paste0("  - \"", categories, "\"", collapse = "\n")
    )
    content <- glue::glue(
        "---\n",
        "title: \"{title}\"\n",
        "description: \"{description}\"\n",
        "{categories_block}\n",
        "---\n\n",
        "Body text.\n"
    )
    readr::write_file(content, path)
}

# Helper: write a minimal .html file to a temp location
make_test_html <- function(path) {
    readr::write_file(
        "<html><body><p>Test content</p></body></html>",
        path
    )
}

# -- .extract_qmd_metadata() --------------------------------------------------

test_that(".extract_qmd_metadata() extracts title correctly", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    make_test_qmd(tmp, title = "My Title")
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_title, "My Title")
})

test_that(".extract_qmd_metadata() extracts description as kb_summary", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    make_test_qmd(tmp, description = "A short summary")
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_summary, "A short summary")
})

test_that(".extract_qmd_metadata() collapses categories into kb_keywords", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    make_test_qmd(tmp, categories = c("reproducibility", "quarto", "R"))
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_keywords, "reproducibility, quarto, R")
})

test_that(".extract_qmd_metadata() returns empty string for missing title", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    readr::write_file("---\ndescription: \"desc\"\n---\n\nBody.\n", tmp)
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_title, "")
})

test_that(".extract_qmd_metadata() returns empty string for missing description", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    readr::write_file("---\ntitle: \"title\"\n---\n\nBody.\n", tmp)
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_summary, "")
})

test_that(".extract_qmd_metadata() returns empty string for missing categories", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    readr::write_file("---\ntitle: \"title\"\n---\n\nBody.\n", tmp)
    meta <- .extract_qmd_metadata(tmp)
    expect_equal(meta$kb_keywords, "")
})

test_that(".extract_qmd_metadata() returns a list with correct names", {
    tmp <- withr::local_tempfile(fileext = ".qmd")
    make_test_qmd(tmp)
    meta <- .extract_qmd_metadata(tmp)
    expect_named(meta, c("kb_title", "kb_summary", "kb_keywords"))
})

# -- generate_kb_xml() validation ---------------------------------------------

test_that("errors informatively when html_path does not exist", {
    expect_error(
        generate_kb_xml(html_path = "/no/such/file.html"),
        "does not exist"
    )
})

test_that("errors informatively when qmd_path cannot be inferred", {
    tmp_html <- withr::local_tempfile(fileext = ".html")
    make_test_html(tmp_html)
    expect_error(
        generate_kb_xml(html_path = tmp_html),
        "Could not find"
    )
})

test_that("errors informatively when explicit qmd_path does not exist", {
    tmp_html <- withr::local_tempfile(fileext = ".html")
    make_test_html(tmp_html)
    expect_error(
        generate_kb_xml(
            html_path = tmp_html,
            qmd_path  = "/no/such/file.qmd"
        ),
        "Could not find"
    )
})

test_that("infers qmd_path correctly from html_path", {
    tmp_dir  <- withr::local_tempdir()
    tmp_html <- fs::path(tmp_dir, "analysis.html")
    tmp_qmd  <- fs::path(tmp_dir, "analysis.qmd")
    make_test_html(tmp_html)
    make_test_qmd(tmp_qmd)

    # function will fail at render step but qmd inference must succeed first
    expect_error(
        generate_kb_xml(html_path = tmp_html),
        regexp = NA
    ) |> tryCatch(error = function(e) {
        expect_false(grepl("Could not find", conditionMessage(e)))
    })
})

test_that("output_dir defaults to directory of html_path", {
    tmp_dir  <- withr::local_tempdir()
    tmp_html <- fs::path(tmp_dir, "analysis.html")
    tmp_qmd  <- fs::path(tmp_dir, "analysis.qmd")
    make_test_html(tmp_html)
    make_test_qmd(tmp_qmd)

    expected_output <- fs::path(tmp_dir, "analysis.xml")

    # intercept before render
    tryCatch(
        generate_kb_xml(html_path = tmp_html),
        error = function(e) NULL
    )

    # if render happened to succeed in env with quarto, check output location
    if (fs::file_exists(expected_output)) {
        expect_true(fs::file_exists(expected_output))
    } else {
        skip("Quarto not available — output_dir defaulting not verified")
    }
})

# -- integration test (requires Quarto) ---------------------------------------

test_that("produces a valid XML file with correct structure", {
    skip_if_not(
        isTRUE(tryCatch(
            { quarto::quarto_version(); TRUE },
            error = function(e) FALSE
        )),
        "Quarto not available"
    )

    tmp_dir  <- withr::local_tempdir()
    tmp_html <- fs::path(tmp_dir, "analysis.html")
    tmp_qmd  <- fs::path(tmp_dir, "analysis.qmd")

    make_test_html(tmp_html)
    make_test_qmd(tmp_qmd,
                  title       = "Integration Title",
                  description = "Integration summary",
                  categories  = c("test", "xml"))

    result <- generate_kb_xml(
        html_path  = tmp_html,
        qmd_path   = tmp_qmd,
        output_dir = tmp_dir
    )

    expect_true(fs::file_exists(result))

    xml_doc  <- xml2::read_xml(result)
    kb_doc   <- xml2::xml_find_first(xml_doc, "kb_document")

    expect_equal(
        xml2::xml_text(xml2::xml_find_first(kb_doc, "kb_title")),
        "Integration Title"
    )
    expect_equal(
        xml2::xml_text(xml2::xml_find_first(kb_doc, "kb_summary")),
        "Integration summary"
    )
    expect_equal(
        xml2::xml_text(xml2::xml_find_first(kb_doc, "kb_keywords")),
        "test, xml"
    )
})

# R/detect_execution_context.R

#' Detect the current execution context
#'
#' Identifies which of three execution environments the code is currently
#' running in: an interactive R session, a `quarto render` call, or a
#' plain `Rscript` invocation. This is useful for writing code that behaves
#' correctly across all three contexts, such as resolving input file paths
#' in a portable way.
#'
#' @param interactive_fn A function. Used to detect whether the session is
#'   interactive. Defaults to `base::interactive`. Override in tests to
#'   simulate different execution environments.
#'
#' @return A character string, one of `"interactive"`, `"quarto"`, or
#'   `"rscript"`.
#'
#' @details
#' Detection follows a priority order:
#'
#' 1. If `interactive()` is `TRUE`, returns `"interactive"`.
#' 2. If the environment variable `QUARTO_DOCUMENT_PATH` is set and non-empty,
#'    returns `"quarto"`.
#' 3. Otherwise, returns `"rscript"`.
#'
#' @export
#'
#' @examples
#' \donttest{
#' context <- detect_execution_context()
#'
#' input_file <- switch(context,
#'   interactive = "data/sample.csv",
#'   quarto      = params$input_file,
#'   rscript     = commandArgs(trailingOnly = TRUE)[1]
#' )
#' }
detect_execution_context <- function(interactive_fn = interactive) {
    if (interactive_fn()) {
        return("interactive")
    }

    if (nchar(Sys.getenv("QUARTO_DOCUMENT_PATH")) > 0) {
        return("quarto")
    }

    return("rscript")
}

# tests/testthat/test-detect_execution_context.R

test_that("returns 'rscript' when not interactive and QUARTO_DOCUMENT_PATH is unset", {
    withr::with_envvar(
        c(QUARTO_DOCUMENT_PATH = ""),
        {
            result <- detect_execution_context(interactive_fn = function() FALSE)
            expect_equal(result, "rscript")
        }
    )
})

test_that("returns 'quarto' when QUARTO_DOCUMENT_PATH is set", {
    withr::with_envvar(
        c(QUARTO_DOCUMENT_PATH = "/some/path/document.qmd"),
        {
            result <- detect_execution_context(interactive_fn = function() FALSE)
            expect_equal(result, "quarto")
        }
    )
})

test_that("returns 'interactive' when session is interactive", {
    result <- detect_execution_context(interactive_fn = function() TRUE)
    expect_equal(result, "interactive")
})

test_that("'interactive' takes priority over QUARTO_DOCUMENT_PATH being set", {
    withr::with_envvar(
        c(QUARTO_DOCUMENT_PATH = "/some/path/document.qmd"),
        {
            result <- detect_execution_context(interactive_fn = function() TRUE)
            expect_equal(result, "interactive")
        }
    )
})

test_that("returns a single character string", {
    withr::with_envvar(
        c(QUARTO_DOCUMENT_PATH = ""),
        {
            result <- detect_execution_context(interactive_fn = function() FALSE)
            expect_type(result, "character")
            expect_length(result, 1)
        }
    )
})

---
title: "Your Document Title"
subtitle: "Your Subtitle"
author:
  - name: "Your Name"
    affiliation: "Your Institution"
    orcid: "Your ORCID"
    email: "your.email@institution.edu"
date: today
date-modified: last-modified
description: "A brief description of this document"
abstract: |
  A longer summary of the document's purpose and contents.
  This can span multiple lines.
categories:
  - "add"
  - "your"
  - "categories"
format:
  html:
    toc: true
    toc-depth: 3
    toc-title: "Contents"
    number-sections: true
    embed-resources: false
    css: assets/styles.css
    include-before-body: assets/header.html
params:
  input_file: "data/sample.csv"
---

```{r}
#| label: setup
#| include: false

# ── Getting started ────────────────────────────────────────────────────────
#
# Welcome! Before you start working on your document, take a moment to
# update the YAML header above with your own information.
#
# Required fields:
#   - title       : replace "Your Document Title" with your actual title
#   - author/name : replace "Your Name" with your name
#   - author/email: replace with your institutional email
#
# Optional but recommended:
#   - subtitle    : a short descriptor beneath the title
#   - affiliation : your department or institution
#   - orcid       : your ORCID identifier (https://orcid.org)
#   - description : one sentence summary, used in search and previews
#   - abstract    : longer summary of purpose and contents
#   - categories  : keywords that describe your document
#
# Fields you don't need:
#   Delete any YAML fields that are not relevant to your work.
#   Quarto will ignore unknown fields, but keeping unused ones
#   adds noise. When in doubt, less is more.
#
# ──────────────────────────────────────────────────────────────────────────

library(readr)
library(dplyr)
library(ggplot2)
library(toolero)
```

## Input Resolution

```{r}
#| label: input-resolution

# detect_execution_context() identifies which of three environments this
# code is running in: interactive RStudio, quarto render, or Rscript.
# Each context resolves the input file path differently.
#
#   interactive : you are running chunks in RStudio — path is hardcoded
#                 to the local data file for development convenience
#   quarto      : document is being rendered — path comes from params
#                 defined in the YAML header above
#   rscript     : document is running on a remote cluster — path comes
#                 from the first command line argument

context <- detect_execution_context()

input_file <- switch(context,
  interactive = "data/sample.csv",
  quarto      = params$input_file,
  rscript     = commandArgs(trailingOnly = TRUE)[1]
)
```

## Load Data

```{r}
#| label: load-data

# read_clean_csv() reads the file and standardizes column names
# using janitor::clean_names() — spaces and special characters
# are replaced with underscores and names are lowercased.

penguins <- read_clean_csv(input_file)

glimpse(penguins)
```

## Analysis

```{r}
#| label: summary

# A simple grouped summary across species.
# Replace this block with your own analysis.

penguins |>
  group_by(species) |>
  summarise(
    n            = n(),
    mean_mass    = mean(body_mass_g, na.rm = TRUE),
    mean_flipper = mean(flipper_length_mm, na.rm = TRUE)
  )
```

```{r}
#| label: plot

# Relationship between flipper length and body mass by species.
# Replace this block with your own plot.

penguins |>
  ggplot(aes(x = flipper_length_mm, y = body_mass_g, color = species)) +
  geom_point(alpha = 0.7) +
  geom_smooth(method = "lm", se = FALSE) +
  labs(
    title    = "Flipper Length vs Body Mass",
    subtitle = "By species — Palmer Penguins dataset",
    x        = "Flipper length (mm)",
    y        = "Body mass (g)",
    color    = "Species"
  ) +
  theme_minimal()
```

---
title: "Getting started with toolero"
author: "Erwin Lares"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with toolero}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## What is toolero?

`toolero` is a small, opinionated toolkit designed to make the first steps of 
an R project faster and more consistent. It targets researchers and analysts 
who want to spend less time on setup and more time on the work itself.

The package currently provides two functions:

- `init_project()` — creates a new R project with a standard folder structure,
  and optionally initializes `renv` and `git`
- `read_clean_csv()` — reads a CSV file and cleans column names in one step

Both functions are designed around a simple idea: the decisions you make at the 
start of a project — how it is organized, how data is read in, how dependencies 
are tracked — have an outsized effect on how maintainable and reproducible that 
project turns out to be. `toolero` tries to make the right defaults easy to 
reach for.

---

## Installation

You can install `toolero` from CRAN:

```{r}
#| eval: false
install.packages("toolero")
```

Or install the development version from GitHub:

```{r}
#| eval: false
pak::pak("erwinlares/toolero")
```

---

## Starting a project with `init_project()`

### The problem

Starting a new R project usually means the same manual steps every time: 
create a folder, set up an RStudio project, create subdirectories for data and 
scripts, initialize `renv`, initialize `git`. None of these steps is hard on 
its own, but skipping any of them — especially early on — tends to create 
friction later. A project without `renv` is harder to share. A project without 
`git` is harder to recover. A project without a consistent folder structure is 
harder to hand off.

### The solution

`init_project()` handles all of this in a single call:

```{r}
#| eval: false
library(toolero)

init_project("~/Documents/my-project")
```

This creates a new RStudio project at the specified path with the following 
folder structure already in place:

```
my-project/
├── data/         # input data
├── data-raw/     # original, unprocessed data
├── R/            # reusable functions
├── scripts/      # analysis scripts
├── plots/        # generated visualizations
├── images/       # static images and assets
├── results/      # processed outputs and tables
└── docs/         # notes, manuscripts, Quarto documents
```

> **Why this structure?** The folder layout is opinionated but not arbitrary.
> Separating `data/` from `data-raw/` makes it clear which files are original
> and which have been processed. Keeping `R/` distinct from `scripts/`
> encourages moving reusable logic into functions over time, which is a natural
> step toward more maintainable code.

By default, `init_project()` also initializes `renv` and `git` in the new 
project. This means the project is reproducible and version-controlled from 
the first commit.

> **Why `renv` and `git` by default?** `renv` ensures that the packages your
> project depends on are recorded and reproducible — someone else (or your
> future self) can restore the exact same environment. `git` provides a full
> history of changes, making it possible to recover from mistakes and understand
> how the project evolved. Both are much easier to set up at the start than to
> retrofit later.

### Adding extra folders

If your project needs folders beyond the defaults, pass them as a character 
vector via `extra_folders`:

```{r}
#| eval: false
init_project(
  "~/Documents/my-project",
  extra_folders = c("notebooks", "presentations")
)
```

### Opting out of renv or git

If you need to skip one or both:

```{r}
#| eval: false
init_project(
  "~/Documents/my-project",
  use_renv = FALSE,
  use_git  = FALSE
)
```

> **When might you skip `renv` or `git`?** Skipping them is occasionally useful
> in teaching or demonstration contexts where the overhead of a full setup is
> unnecessary. For any project you plan to share, archive, or return to later,
> the defaults are strongly recommended.

---

## Reading data with `read_clean_csv()`

### The problem

Reading a CSV file into R is straightforward — until the column names come back 
with spaces, mixed capitalization, or special characters. Cleaning them up is a 
small but recurring friction point.

### The solution

`read_clean_csv()` combines `readr::read_csv()` and `janitor::clean_names()` 
into a single call:

```{r}
#| eval: false
data <- read_clean_csv("data/my-file.csv")
```

Column names are automatically converted to lowercase with underscores — 
consistent, predictable, and tidyverse-friendly. A column called 
`First Name` becomes `first_name`. `Q1 Revenue ($)` becomes `q1_revenue`.

By default, column type messages from `readr` are suppressed to keep the 
output clean. If you want to see them — useful when reading an unfamiliar 
dataset for the first time — set `verbose = TRUE`:

```{r}
#| eval: false
data <- read_clean_csv("data/my-file.csv", verbose = TRUE)
```

> **Why suppress messages by default?** Column type messages are helpful when
> you are first exploring a dataset. In a script or document that runs
> repeatedly, they become noise. The `verbose` argument gives you control
> without requiring you to remember the `readr` option name.

---

## What's next for toolero?

`toolero` is intentionally small right now. The two functions it provides solve 
a specific, recurring problem — getting a project started correctly. Future 
versions may include:

- Utilities for common data validation patterns
- Helpers for writing reproducible reports
- Functions that ease the transition from local analysis to running code on 
  remote computing infrastructure

The goal is not to be comprehensive. It is to make the right habits easy to 
reach for from the first line of code.

---

## Citation

If you use `toolero` in your work, please cite it:

```{r}
#| eval: false
citation("toolero")
```

---
title: "Getting started with toolero"
author: "Erwin Lares"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with toolero}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Background and motivation

`toolero` grew out of a recurring observation made while teaching and
supporting researchers at UW-Madison: the habits that make a project
reproducible, shareable, and maintainable are easiest to adopt at the very
beginning — and hardest to retrofit once a project is already underway.

The package is heavily influenced by the workflows taught in workshops run by
[The Carpentries](https://carpentries.org/) and the
[UW-Madison Libraries](https://www.library.wisc.edu/). Those workshops
emphasize consistent project organization, version control, and reproducible
data practices as foundational skills — not advanced topics. `toolero` tries
to operationalize those principles into a small set of functions that reduce
the friction of doing the right thing from the start.

The theming and branding support in `toolero` is specifically tailored to
UW-Madison's Research Computing and Instrumentation (RCI) unit, whose
Quarto-based reporting templates are baked into the package as defaults. If
you are not at UW-Madison, the branding files are optional — the rest of the
package works independently of them.

---

## Who is this for?

`toolero` is designed for researchers and analysts who:

- Work primarily in R and use RStudio as their IDE
- Write reports or analyses in Quarto
- Want consistent, reproducible project structure without having to think
  about it every time
- May need to publish content to the UW-Madison Knowledge Base

The package is intentionally small. It does not try to be comprehensive. It
tries to make the right defaults easy to reach for from the first line of
code.

---

## Installation

You can install `toolero` from CRAN:

```{r}
#| eval: false
install.packages("toolero")
```

Or install the development version from GitHub:

```{r}
#| eval: false
pak::pak("erwinlares/toolero")
```

---

## Project setup: `init_project()` and `create_qmd()`

These two functions are designed to be used together, in order. `init_project()`
creates the scaffold; `create_qmd()` populates it with a working Quarto
document.

### Starting with `init_project()`

Starting a new R project usually means the same manual steps every time:
create a folder, set up an RStudio project, create subdirectories for data
and scripts, initialize `renv`, initialize `git`. None of these steps is hard
on its own, but skipping any of them — especially early on — tends to create
friction later.

`init_project()` handles all of this in a single call:

```{r}
#| eval: false
library(toolero)

init_project(path = "~/Documents/my-project")
```

This creates a new RStudio project at the specified path with the following
folder structure already in place:

```
my-project/
├── data/         # input data
├── data-raw/     # original, unprocessed data
├── R/            # reusable functions
├── scripts/      # analysis scripts
├── plots/        # generated visualizations
├── images/       # static images and assets
├── results/      # processed outputs and tables
└── docs/         # notes, manuscripts, Quarto documents
```

> **Why this structure?** The folder layout is opinionated but not arbitrary.
> Separating `data/` from `data-raw/` makes it clear which files are original
> and which have been processed. Keeping `R/` distinct from `scripts/`
> encourages moving reusable logic into functions over time, which is a natural
> step toward more maintainable code.

By default, `init_project()` also initializes `renv` and `git`. This means
the project is reproducible and version-controlled from the first commit.

> **Why `renv` and `git` by default?** `renv` ensures that the packages your
> project depends on are recorded and reproducible. `git` provides a full
> history of changes. Both are much easier to set up at the start than to
> retrofit later.

If your project needs folders beyond the defaults:

```{r}
#| eval: false
init_project(
  path          = "~/Documents/my-project",
  extra_folders = c("notebooks", "presentations")
)
```

To apply UW-Madison RCI branding assets to the project:

```{r}
#| eval: false
init_project(
  path        = "~/Documents/my-project",
  uw_branding = TRUE
)
```

This creates an `assets/` folder and populates it with `styles.css`,
`header.html`, and `rci-banner.png` — the same assets used in the Quarto
template scaffolded by `create_qmd()`.

### Adding a Quarto document with `create_qmd()`

Once the project exists, `create_qmd()` adds a working Quarto document to it:

```{r}
#| eval: false
create_qmd(path = "~/Documents/my-project", filename = "analysis.qmd")
```

This creates:

- `analysis.qmd` — a Quarto document with a fully populated YAML header,
  three-context input resolution via `detect_execution_context()`, and a
  sample analysis using the Palmer Penguins dataset
- `data/sample.csv` — sample data to develop against immediately
- `assets/styles.css` and `assets/header.html` — UW-Madison RCI branding
- `_quarto.yml` — a project file with a post-render hook that runs `purl.R`
- `purl.R` — extracts R code from the rendered document into a companion
  `.R` file automatically on every render

> **Why the purl hook?** Having a plain `.R` companion to your `.qmd` is
> useful for sharing the analysis as a script, running it on a remote cluster,
> or archiving the code independently of the document. The hook runs
> automatically so you never have to remember to extract it manually.

To pre-populate the YAML header with your own metadata:

```{r}
#| eval: false
create_qmd(
  path      = "~/Documents/my-project",
  filename  = "analysis.qmd",
  yaml_data = "~/my-metadata.yml"
)
```

Where `my-metadata.yml` might look like:

```yaml
title: "My Analysis"
author:
  - name: "Your Name"
    affiliation: "UW-Madison"
    email: "you@wisc.edu"
```

Any keys present in the YAML file overwrite the corresponding placeholders in
the template. Keys not present are left as-is.

---

## Working with data: `read_clean_csv()` and `write_by_group()`

These two functions address common friction points in day-to-day data work.
They are general-purpose utilities — useful in any R project, not just ones
set up with `toolero`.

### Reading data with `read_clean_csv()`

`read_clean_csv()` combines `readr::read_csv()` and `janitor::clean_names()`
into a single call:

```{r}
#| eval: false
data <- read_clean_csv("data/my-file.csv")
```

Column names are automatically converted to lowercase with underscores —
consistent, predictable, and tidyverse-friendly. A column called `First Name`
becomes `first_name`. `Q1 Revenue ($)` becomes `q1_revenue`.

By default, column type messages from `readr` are suppressed. Set
`verbose = TRUE` to see them:

```{r}
#| eval: false
data <- read_clean_csv("data/my-file.csv", verbose = TRUE)
```

### Splitting data by group with `write_by_group()`

When a data frame contains multiple groups that need to be written to separate
files, `write_by_group()` handles the split and the write in a single call:

```{r}
#| eval: false
write_by_group(
  data       = penguins,
  group_col  = "species",
  output_dir = "results/by-species"
)
```

Output filenames are derived from the group values and sanitized for use as
file names — converted to lowercase with spaces and special characters
replaced by dashes. A group called `Chinstrap` becomes `chinstrap.csv`.
`Palmer Penguins` would become `palmer-penguins.csv`.

To also write a manifest listing the output files, group values, and row
counts:

```{r}
#| eval: false
write_by_group(
  data       = penguins,
  group_col  = "species",
  output_dir = "results/by-species",
  manifest   = TRUE
)
```

---

## Execution context: `detect_execution_context()`

R code often needs to behave differently depending on where it is running —
interactively in RStudio, during a `quarto render`, or as a batch `Rscript`
job on a remote cluster. `detect_execution_context()` identifies which of
these three environments is active and returns one of `"interactive"`,
`"quarto"`, or `"rscript"`.

The canonical use case is resolving input file paths portably:

```{r}
#| eval: false
context <- detect_execution_context()

input_file <- switch(context,
  interactive = "data/sample.csv",
  quarto      = params$input_file,
  rscript     = commandArgs(trailingOnly = TRUE)[1]
)
```

This pattern is built into the template scaffolded by `create_qmd()`, so you
get it for free without having to write it yourself.

---

## Knowledge Base export: `generate_kb_xml()`

> **This section is relevant only if you publish content to the UW-Madison
> Knowledge Base.** If you do not, you can safely skip it.

The UW-Madison Knowledge Base requires content to be submitted as XML with
all visual assets embedded in the HTML body. `generate_kb_xml()` automates
this process entirely.

```{r}
#| eval: false
generate_kb_xml(
  html_path  = "docs/analysis.html",
  output_dir = "exports"
)
```

The function:

1. Infers the source `.qmd` from the HTML path (or accepts it explicitly via
   `qmd_path`)
2. Re-renders the document with `embed-resources: true` so all CSS, images,
   and JavaScript are self-contained
3. Extracts metadata from the `.qmd` YAML header — `title` → `kb_title`,
   `description` → `kb_summary`, `categories` → `kb_keywords`
4. Produces a `.xml` file ready for direct KB import

This is why the `description` and `categories` fields in the `create_qmd()`
template matter — they flow through automatically into the KB article metadata
without any extra work.

> **When importing into the KB**, check the
> *Decode HTML entity in body content* option.

---

## Citation

If you use `toolero` in your work, please cite it:

```{r}
#| eval: false
citation("toolero")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write-by-group.R
\name{write_by_group}
\alias{write_by_group}
\title{Split a data frame by a grouping column and write each group to a CSV file}
\usage{
write_by_group(data, group_col, output_dir = NULL, manifest = FALSE)
}
\arguments{
\item{data}{A data frame or tibble to split and save.}

\item{group_col}{A string. The name of the column to group by.}

\item{output_dir}{A string or \code{NULL}. Path to the directory where output
files will be written. Created if it does not exist. If \code{NULL}, the
user must supply a path explicitly.}

\item{manifest}{A logical. Whether to write a \code{manifest.csv} file to
\code{output_dir} listing the output files, group values, and row counts.
Defaults to \code{FALSE}.}
}
\value{
Invisibly returns \code{output_dir}.
}
\description{
Splits a data frame by a single grouping column and writes each group to
a separate CSV file. Optionally writes a manifest file listing the output
files, their group values, and row counts.
}
\details{
Output filenames are derived from the group values of \code{group_col}.
Values are sanitized before use as filenames: converted to lowercase,
spaces and special characters replaced with \verb{_}, consecutive underscores
collapsed, and leading/trailing underscores stripped.

If \code{manifest = TRUE}, a \code{manifest.csv} is written to \code{output_dir}
containing three columns: \code{group_value}, \code{n_rows}, and \code{file_path}.

Note: \code{output_dir} has no default value. Always supply an explicit path
to avoid writing files to unexpected locations. Use \code{tempdir()} for
temporary output during testing or exploration.
}
\examples{
\donttest{
# Split a small data frame by group and write to a temp directory
data <- data.frame(
  species = c("Adelie", "Adelie", "Gentoo"),
  mass    = c(3750, 3800, 5000)
)
write_by_group(data, group_col = "species", output_dir = tempdir())

# Same but also write a manifest
write_by_group(data, group_col = "species",
               output_dir = tempdir(), manifest = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/init-project.R
\name{init_project}
\alias{init_project}
\title{Initialize a new R project with a standard folder structure}
\usage{
init_project(
  path,
  use_renv = TRUE,
  use_git = TRUE,
  extra_folders = NULL,
  open = FALSE,
  uw_branding = FALSE
)
}
\arguments{
\item{path}{A character string with the path and name of the new
project (e.g., \code{"~/Documents/my-project"}).}

\item{use_renv}{Logical. If \code{TRUE}, initializes \code{renv} in the new project.
Defaults to \code{TRUE}.}

\item{use_git}{Logical. If \code{TRUE}, initializes a git repository in the new
project. Defaults to \code{TRUE}.}

\item{extra_folders}{A character vector of additional folder names to create
inside the project. Defaults to \code{NULL}.}

\item{open}{Logical. If \code{TRUE}, opens the new project in RStudio after
creation. Defaults to \code{TRUE}.}

\item{uw_branding}{Logical. If \code{TRUE}, creates an \verb{assets/} folder and
populates it with UW-Madison RCI branding files (\code{styles.css},
\code{header.html}, \code{rci-banner.png}). Defaults to \code{FALSE}.}
}
\value{
Called for its side effects. Does not return a value.
}
\description{
\code{init_project()} creates a new R project at the given path with an
opinionated folder structure suited for research workflows. It optionally
initializes \code{renv} for package management and git for version control.
}
\examples{
\dontrun{
init_project(path = file.path(tempdir(), "project1"),
             use_renv = FALSE, use_git = FALSE)

init_project(path = file.path(tempdir(), "project2"),
             uw_branding = TRUE, use_renv = FALSE, use_git = FALSE)

init_project(path = file.path(tempdir(), "project3"),
             extra_folders = c("notebooks"),
             use_renv = FALSE, use_git = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create-qmd.R
\name{create_qmd}
\alias{create_qmd}
\title{Create a new Quarto document from a template}
\usage{
create_qmd(
  filename = NULL,
  path = ".",
  yaml_data = NULL,
  overwrite = FALSE,
  use_purl = TRUE
)
}
\arguments{
\item{filename}{A string or \code{NULL}. Name of the generated \code{.qmd} file.
Must be supplied explicitly, e.g. \code{"analysis.qmd"}.}

\item{path}{A string. Path to the directory where the document will be
created. Defaults to \code{"."} (the current working directory).}

\item{yaml_data}{A string or \code{NULL}. Path to a YAML file containing
metadata to pre-populate the document header. If \code{NULL} (the default),
the template is copied as-is with placeholder prompts intact.}

\item{overwrite}{A logical. Whether to overwrite existing files. Defaults
to \code{FALSE}.}

\item{use_purl}{Logical. If \code{TRUE} (the default), creates a \verb{_quarto.yml}
file with a post-render hook and a \code{purl.R} script inside \verb{R/} that
extracts R code from the rendered document into a \code{.R} file. The target
document is resolved dynamically by scanning the project root for \code{.qmd}
files, so the same \code{purl.R} works regardless of the document name.}
}
\value{
Invisibly returns \code{path}.
}
\description{
Creates a new Quarto document in the specified directory, along with a
sample dataset and UW-Madison branded assets. Optionally pre-populates
the YAML header with user-supplied metadata.
}
\details{
\code{create_qmd()} performs the following steps:
\enumerate{
\item Validates that \code{path} exists.
\item Validates that \code{filename} is supplied.
\item Creates a \verb{data/} folder under \code{path} and copies \code{sample.csv} there.
\item Checks for \code{assets/styles.css} and \code{assets/header.html} - creates the
\verb{assets/} folder if needed and copies both from the package.
\item Copies the template \code{.qmd} to \code{path/filename}.
\item If \code{yaml_data} is provided, reads the YAML file and substitutes values
into the document header.
\item If \code{use_purl = TRUE}, writes a \verb{_quarto.yml} with a post-render hook
pointing to \code{R/purl.R}, and copies \code{purl.R} from the package templates
into \code{path/R/purl.R}.
}

Note: \code{filename} has no default value and must always be supplied
explicitly. Use \code{tempdir()} for temporary output during testing or
exploration.
}
\examples{
\donttest{
# Create a document in a temp directory
create_qmd(path = tempdir(), filename = "analysis.qmd")

# Create with a custom filename, without the purl hook
create_qmd(path = tempdir(), filename = "report.qmd",
            overwrite = TRUE, use_purl = FALSE)

# Create with pre-populated YAML
yaml_file <- tempfile(fileext = ".yml")
writeLines("author:\n  - name: 'Your Name'", yaml_file)
create_qmd(path = tempdir(), filename = "analysis.qmd",
            yaml_data = yaml_file, overwrite = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-clean-csv.R
\name{read_clean_csv}
\alias{read_clean_csv}
\title{Read and clean a CSV file}
\usage{
read_clean_csv(file_path, verbose = FALSE)
}
\arguments{
\item{file_path}{A character string with the path to the CSV file.}

\item{verbose}{Logical. If \code{TRUE}, displays column type messages from
\code{readr::read_csv()}. Defaults to \code{FALSE}.}
}
\value{
A tibble with clean column names.
}
\description{
\code{read_clean_csv()} reads a CSV file and cleans the column names in one step.
It leverages \code{readr::read_csv()} for reading and \code{janitor::clean_names()} for
making column names tidyverse-friendly (lowercase, no spaces, no special
characters). By default, column type messages are suppressed. Set
\code{verbose = TRUE} to display them.
}
\examples{
\donttest{
# Read and clean a CSV file silently
sample_path <- system.file("templates", "sample.csv", package = "toolero")
data <- read_clean_csv(sample_path)

# Show column type messages
data <- read_clean_csv(sample_path, verbose = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect-execution-context.R
\name{detect_execution_context}
\alias{detect_execution_context}
\title{Detect the current execution context}
\usage{
detect_execution_context(interactive_fn = interactive)
}
\arguments{
\item{interactive_fn}{A function. Used to detect whether the session is
interactive. Defaults to \code{base::interactive}. Override in tests to
simulate different execution environments.}
}
\value{
A character string, one of \code{"interactive"}, \code{"quarto"}, or
\code{"rscript"}.
}
\description{
Identifies which of three execution environments the code is currently
running in: an interactive R session, a \verb{quarto render} call, or a
plain \code{Rscript} invocation. This is useful for writing code that behaves
correctly across all three contexts, such as resolving input file paths
in a portable way.
}
\details{
Detection follows a priority order:
\enumerate{
\item If \code{interactive()} is \code{TRUE}, returns \code{"interactive"}.
\item If the environment variable \code{QUARTO_DOCUMENT_PATH} is set and non-empty,
returns \code{"quarto"}.
\item Otherwise, returns \code{"rscript"}.
}
}
\examples{
\donttest{
context <- detect_execution_context()

input_file <- switch(context,
  interactive = "data/sample.csv",
  quarto      = params$input_file,
  rscript     = commandArgs(trailingOnly = TRUE)[1]
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate-kb-xml.R
\name{generate_kb_xml}
\alias{generate_kb_xml}
\title{Generate a KB-importable XML file from a Quarto document}
\usage{
generate_kb_xml(html_path, qmd_path = NULL, output_dir = NULL)
}
\arguments{
\item{html_path}{A string. Path to the rendered HTML file. Used to infer
the output filename and, if \code{qmd_path} is \code{NULL}, the location of the
source \code{.qmd}.}

\item{qmd_path}{A string or \code{NULL}. Path to the source \code{.qmd} file. If
\code{NULL} (the default), inferred by replacing the \code{.html} extension of
\code{html_path} with \code{.qmd}.}

\item{output_dir}{A string or \code{NULL}. Directory where the \code{.xml} file
will be written. If \code{NULL} (the default), written to the same directory
as \code{html_path}.}
}
\value{
Invisibly returns the path to the written \code{.xml} file.
}
\description{
Takes a Quarto document and produces an XML file that is directly
importable into a UW-Madison Knowledge Base (KB) article. The function
re-renders the \code{.qmd} with \code{embed-resources: true} so all visual assets
are self-contained, extracts the HTML body, and wraps it in the KB XML
structure along with metadata drawn from the document's YAML header.
}
\details{
\code{generate_kb_xml()} performs the following steps:
\enumerate{
\item Validates that \code{html_path} exists.
\item Infers \code{qmd_path} from \code{html_path} if not supplied, then validates it.
\item Extracts \code{title}, \code{description}, and \code{categories} from the \code{.qmd} YAML
header and maps them to \code{kb_title}, \code{kb_summary}, and \code{kb_keywords}.
\item Re-renders the \code{.qmd} in an isolated temporary directory with
\code{embed-resources: true} so all CSS, images, and JS are self-contained.
The \verb{data/} and \verb{assets/} folders are copied alongside the \code{.qmd} to
ensure the render succeeds.
\item Extracts the \verb{<body>} from the embedded HTML.
\item Escapes HTML entities in the body for XML compatibility, as required
by the UW-Madison KB import format.
\item Builds the XML structure with \code{kb_title}, \code{kb_keywords}, \code{kb_summary},
and \code{kb_body} nodes.
\item Writes the \code{.xml} file to \code{output_dir}.
}

Temporary files are managed via \code{withr::local_tempdir()} and are
automatically cleaned up when the function exits, even on error.

When importing the resulting XML into the KB, check the
\emph{Decode HTML entity in body content} option.
}
\examples{
\donttest{
# Infer qmd_path automatically, write XML alongside the HTML
# generate_kb_xml(html_path = "docs/analysis.html")

# Supply qmd_path explicitly and write to a specific output directory
# generate_kb_xml(
#   html_path  = "docs/analysis.html",
#   qmd_path   = "analysis.qmd",
#   output_dir = "exports"
# )
}
}
